package com.fuzamei.service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fuzamei.constant.OperationTypeId;
import com.fuzamei.constant.Role;
import com.fuzamei.dao.ProjectFileDao;
import com.fuzamei.entity.ProjectFile;
import com.fuzamei.entity.UserDetail;
import com.fuzamei.utils.PageDTO;

@Service
public class ProjectFileService {

	@Autowired
	private ProjectFileDao projectFileDao;
	
	@Autowired
	private UserAuthoricationService userAuthoricationService;

	/**
	 * 
	* @Title: queryProjectFileInformation
	* @Description: TODO(查询项目文档的信息)
	* @param @param map
	* @param @return    设定文件
	* @return PageDTO    返回类型
	* @author ylx
	* @date 2017年12月15日 下午3:57:43
	* @throws
	 */
	public PageDTO queryProjectFileInformation(Map<String, Object> map) {
		PageDTO pageDto = new PageDTO(); // 创建分页对象
		int userId = (Integer) map.get("userId");
		userAuthoricationService.checkUserAuthority(userId,Role.SUPERVISION_AGENCY,Role.ARCHIVE_KEEPING_POS_OF_PBBTBC);		//查看用户非空和权限
		List<ProjectFile> informationList = projectFileDao.queryProjectFileInformation(map);
		int count = projectFileDao.findAllInformation(map);
		pageDto.setRows(informationList);
		pageDto.setTotal(count);
		return pageDto;
	}

	
	/**
	 * 
	* @Title: checkoutProjectFiles
	* @Description: TODO通过点击查看按钮，可以查看到这个项目名称下所有的项目文件信息以及hash值，并且提供文件的下载功能
	* @param @param map
	* @param @return    设定文件
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月15日 下午4:19:56
	* @throws
	 */
	public Map<String, Object> checkoutProjectFiles(Map<String, Object> map) {
		int userId = (Integer) map.get("userId");
		userAuthoricationService.checkUserAuthority(userId,Role.SUPERVISION_AGENCY,Role.ARCHIVE_KEEPING_POS_OF_PBBTBC);	//查看用户非空和权限
		List<ProjectFile> informationList = projectFileDao.checkoutProjectFiles(map);	//将项目id和项目名称封装成map到dao层去获取数据
		Map<String, Object> mapBackToController = new LinkedHashMap<String, Object>();
		mapBackToController.put("projectId", map.get("projectId"));		//展示给前端的项目id号码
		mapBackToController.put("projectName", map.get("projectName"));	//展示给前端的项目名称
		mapBackToController.put("fileInformations", informationList);	//展示给前端的所有文件信息(包括文件名、文件hash值，还有文件url下载地址)
		return mapBackToController;
	}

	/**
	 * 
	* @Title: insertInformationIntoTable
	* @Description: TODO(上传项目文档，不仅要将数据插入项目文档表和附件表，还要在操作记录中插入一条数据)
	* @param @param map    设定文件
	* @return void    返回类型
	* @author ylx
	* @date 2017年12月20日 下午8:16:14
	* @throws
	 */
	@Transactional(rollbackFor=Exception.class)
	public void insertInformationIntoTable(Map<String, Object> map) {
		long currentTime = System.currentTimeMillis();										//获取当前时间
		int attachmentId = (Integer)(int)(Math.random()*1000000);							//附件id不能用随机数，这里到时候要改的================>>待定TODO
		UserDetail userDetail = (UserDetail) map.get("userDetail");							//获取controller传过来的UserDetail对象
		//将数据封装到mapToDaoForProjectFile，并插入【项目文档表】
		Map<String, Object> mapToDaoForProjectFile = new LinkedHashMap<String,Object>();
		mapToDaoForProjectFile.put("projectId", map.get("projectId"));
		mapToDaoForProjectFile.put("projectName", map.get("projectName"));
		mapToDaoForProjectFile.put("attachmentId", attachmentId);							//附件id不能用随机数，这里到时候要改的================>>待定TODO
		mapToDaoForProjectFile.put("updateTime", currentTime);								//上传文件的更新时间为当前时间
		mapToDaoForProjectFile.put("hash", map.get("hash"));								//将算好的hash值放入mapToDaoForProjectFile中
		projectFileDao.insertInformationIntoTable(mapToDaoForProjectFile);					//将以上的数据直接添加到数据库中【项目文档表】去
		
		//将数据封装到mapToDaoForAttachment，并插入【附件表】
		Map<String, Object> mapToDaoForAttachment = new LinkedHashMap<String,Object>();
		mapToDaoForAttachment.put("attachmentId", attachmentId);							//附件id不能用随机数，这里到时候要改的================>>待定TODO
		mapToDaoForAttachment.put("attachmentName", map.get("attachmentName"));				//附件名称需要录入
		mapToDaoForAttachment.put("url", map.get("url"));									//下载文件需要用到的url地址需要录入
		mapToDaoForAttachment.put("userId", userDetail.getUser_id());						//直接通过userDetail中获取
		mapToDaoForAttachment.put("addTime", currentTime);									//这里添加的文件上传时间，和项目文档更新时间一样
		projectFileDao.insertInformationToAttachment(mapToDaoForAttachment);
		
		//此外，还要将业务申报的这个操作信息插入到操作记录表中【操作记录表】
		Map<String, Object> mapToOperationHistory = new LinkedHashMap<String,Object>();
		mapToOperationHistory.put("operatorTypeId", OperationTypeId.UPLOAD_PROJECT_FILE);//上传项目文档的类型为1011
		mapToOperationHistory.put("operatorAccount", userDetail.getAccount());			//操作账号通过userDetail中获取
		mapToOperationHistory.put("operatorRole", Role.ARCHIVE_KEEPING_POS_OF_PBBTBC);	//创建这个操作的角色只能是省分行托管分中心档案保管岗
		mapToOperationHistory.put("operatorPerson", userDetail.getPerson_name());		//操作人通过userDetail中获取
		mapToOperationHistory.put("operatorTime", currentTime);							//操作时间为当前操作时间
		mapToOperationHistory.put("hash", "来自前端token计算");							//如何计算操作的hash值？(按照上传的项目文件的hash值算)=====>>待定TODO
		projectFileDao.insertOperationHistoryInformationToDao(mapToOperationHistory);   //将操作记录信息插入到操作记录表中
		
	}
	/*public static void main(String[] args) throws NoSuchAlgorithmException {
		MessageDigest complete=MessageDigest.getInstance("MD5");
		byte[] bs={1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
		complete.update(bs);
		byte[] digest = complete.digest();
		System.out.println(Arrays.toString(digest));
		System.out.println("=======================");
		System.out.println(Integer.toString(0x100));
		System.out.println(Integer.toString(36, 37));
		System.out.println(Integer.toString((16 & 0xff) + 0x100, 16).substring( 1 ));
		System.out.println(( 1 & 0xff ) + 0x100);
		System.out.println("=======================");
		
	}*/
	
}
