package com.fuzamei.service;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.annotation.Resource;
import javax.management.RuntimeErrorException;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fuzamei.dao.UserDao;
import com.fuzamei.entity.User;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.blockchain.KeyUtils;
/**
 * 
* <b>Description:用户登录的service层</b><br> 
* @Note
* <b>Author:maamin
* @version 1.0
* @Note
* <b>ProjectName:</b> ccb_fund_trusteeship
* <br><b>PackageName:</b> com.fuzamei.service
* <br><b>ClassName:</b> UserService
* <br><b>Date:</b> 2017年12月25日 上午10:37:17
 */
@Service(value = "userService")
public class UserService {
	@Resource(name = "userDao")
	private UserDao userDao;
	/**
	 * 
	* <b>Description:用户登录方法</b><br> 
	* @param user--用户实体类
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月25日 上午10:36:10
	* <br><b>Version:</b> 1.0
	 */
	public Map<String, Object> login(User user) {
		// TODO 检查用户的用户名、密码和随机数，能不能对应得上
		Map<String,Object> resultMap = new LinkedHashMap<String,Object>();
		int count = userDao.getUserCount(user);
		resultMap = JSONUtil.getJsonMap(12431, false, "用户登录失败", null);
		if(count == 1){
			resultMap = JSONUtil.getJsonMap(200, true, "用户登录成功", null);
		}
		return resultMap;
	}
	/**
	 * 
	* <b>Description:初始化所有用户信息(后台维护人员为了防止在数据库中添加设计，后期废弃掉)</b><br> 
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月25日 上午10:37:46
	* <br><b>Version:</b> 1.0
	 */
	@Transactional
	public Map<String, Object> initUserList() {
		// TODO Auto-generated method stub
		//封装用户信息到listUsers中
		/*List<User> listUsers = UserUtil.getUserList();	
		List<Map<String,Object>> list = UserUtil.getUserRoleList();
		//将用户信息插入到t_user表中
		int insertUserCount = userDao.insertUsers(listUsers);
		if(insertUserCount == 18){	//一共有18个角色
			
		}*/
		return null;
	}
	/**
	 * 
	* <b>Description:添加用户</b><br> 
	* @param user
	* @return
	* @Note
	* <b>Author:maamin
	* <br><b>Date:</b> 2017年12月26日 下午5:45:31
	* <br><b>Version:</b> 1.0
	 */
	@Transactional
	public Map<String, Object> addUser(User user) {
		Map<String,Object> resultMap = new HashMap<String,Object>();
		// TODO 将公私钥生成出来
		String password = user.getPassword();
		String random = user.getRandom();
		String private_key = KeyUtils.getPrivateKey(password, random);
		String public_key = KeyUtils.getPublicKey(private_key);
		user.setPrivate_key(private_key);
		user.setPublic_key(public_key);
		long currentTime = System.currentTimeMillis();
		user.setCreate_time(currentTime);
		user.setUpdate_time(currentTime);
		// TODO 用户添加到本地数据库
		int userCount = userDao.addUserToUser(user);
		int userRoleCount = userDao.addUserToUserRole(user);
		if((userCount == 1)&&(userRoleCount == 1)){
			//区块链操作开始
			//区块链操作结束
			resultMap = JSONUtil.getJsonMap(200, false, "用户添加成功", null);
		}
		else{
			throw new RuntimeException();
		}
		return resultMap;
	}
	@Transactional
	public Map<String, Object> editUser(User user) {
		Map<String,Object> resultMap = new HashMap<String,Object>();
		// TODO Auto-generated method stub
		int countUser = userDao.updateUserById(user);
		int countRole = userDao.updataRoleById(user);
		if((countUser == 1)&&(countRole == 1)){
			resultMap = JSONUtil.getJsonMap(200, false, "用户信息修改成功", null);
		}
		else{
			throw new RuntimeException("blockchain error!");
		}
		return resultMap;
	}
}
