package com.fuzamei.web;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.fuzamei.entity.OperationHistoryBO;
import com.fuzamei.entity.UserDetail;
import com.fuzamei.service.OperationHistoryService;
import com.fuzamei.service.UserAuthoricationService;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.PageDTO;
import com.fuzamei.utils.StringUtil;
import com.fuzamei.utils.ValidationUtil;


@Controller
@RequestMapping("/operation_history")
public class OperationHistoryAction {

	@Autowired
	private OperationHistoryService operationHistoryService;
	
	private static final int ROW_NUM = 10; // 分页每页显示数据的数量
	/**
	 * 
	* @Title: queryOperationHistoryInformation
	* @Description: TODO(查询操作记录信息....)
	* @param @param data--前台传递过来的application/json数据
	* 				传递过来的参数json格式为：
	* 				全部是字符串
					{
						"userId":"xxx",
						"page":"1",			
						"account":"",
						"role":"",
						"type":"",
						"startTime":"",
						"endTime":""
					}
	* @param @return    返回json格式
	* @return Map<String,Object>    返回类型
	* @author ylx
	* @date 2017年12月14日 下午5:12:17
	* @throws
	 */
	@RequestMapping(value="/queryOperationHistoryInformation",method=RequestMethod.POST)
	@ResponseBody
	public Map<String,Object> queryOperationHistoryInformation(@RequestBody String data){
		try {
			System.out.println("查询操作记录信息....");
			@SuppressWarnings("unchecked")
			Map<String, String> map = JSON.parseObject(data, Map.class);	//将前端传过来的json数据转化为map对象，并将map对象传入service层进行数据校验
			int userId = ValidationUtil.checkAndAssignInt(map.get("userId"));	//userId只要是一个int类型即可
			int page = ValidationUtil.checkMinAndAssignInt(map.get("page"), 1); // 默认页是第一页
			String account = ValidationUtil.checkBlankStringAndAssignEmptyIfIsBlank(map.get("account")); // 账号默认空值""（空串，也可以是null,根据需求改）
			String role = ValidationUtil.checkBlankStringAndAssignEmptyIfIsBlank(map.get("role")); // 操作角色默认空值""
			String type = ValidationUtil.checkBlankStringAndAssignEmptyIfIsBlank(map.get("type"));; // 操作类型默认空值""
			long startTime = ValidationUtil.checkAndAssignDefaultLong(map.get("startTime"), 0); // 开始时间默认0
			long endTime = ValidationUtil.checkAndAssignDefaultLong(map.get("endTime"), Long.MAX_VALUE); // 结束时间默认Long最大值
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("userId", userId);					//用户id用于查询权限或者角色信息
			mapToService.put("startPage", (page - 1) * ROW_NUM);
			mapToService.put("account", account);
			mapToService.put("role", role);
			mapToService.put("type", type);
			if (startTime <= endTime) {
				mapToService.put("startTime", startTime);
				mapToService.put("endTime", endTime);
			} else {
				mapToService.put("startTime", startTime);
				mapToService.put("endTime", Long.MAX_VALUE);
			}
			mapToService.put("rowNum", ROW_NUM); //默认每页显示数据是10条，可根据需求修改分页数量
			PageDTO pageDto = operationHistoryService.queryOperationHistoryInformation(mapToService);
			return JSONUtil.getJsonMap(200, true, "查询成功", pageDto);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "查询失败："+e.getMessage(), null);
		}
	}
	
	
	
	/*@RequestMapping(value="/test",method=RequestMethod.POST)
	@ResponseBody
	public Map<String,Object> test(@RequestBody String data){
		int userId=15254122;
		UserDetail detail=operationHistoryService.queryUserDetailInformation(userId);
		System.out.println(detail);
		return JSONUtil.getJsonMap(200, true, "查询成功", null);
	}*/
	
	/*OperationHistoryBO operationHistoryBO = JSON.parseObject(data, OperationHistoryBO.class);
	List<String> validateList = ValidationUtil.validate(operationHistoryBO);
	if(!validateList.isEmpty()){
		throw new RuntimeException(validateList.get(0));	//抛出校验失败异常
	}
	PageDTO pageDto = operationHistoryService.queryOperationHistoryInformation(operationHistoryBO);*/
	
	/*if (StringUtil.isNotEmpty(map.get("userId"))) { 		// 等于空就直接取空值
	try {
		userId = Integer.parseInt((String) map.get("userId"));
	} catch (Exception e) {
		throw new RuntimeException("userId类型转换异常");
	}
}
if (StringUtil.isNotEmpty(map.get("page"))) { 			// 等于空就直接取第一页
	try {
		page = Integer.parseInt((String) map.get("page"));
		if (page < 1) {
			page = 1;
		}
	} catch (Exception e) {
		throw new RuntimeException("page类型转换异常");
	}
}
if (StringUtil.isNotEmpty(map.get("account"))) { 		// 等于空就直接取空值
	try {
		account = (String) map.get("account");
	} catch (Exception e) {
		throw new RuntimeException("account类型转换异常");
	}
}
if (StringUtil.isNotEmpty(map.get("role"))) { 			// 等于空就直接取空值
	try {
		role = (String) map.get("role");
	} catch (Exception e) {
		throw new RuntimeException("role类型转换异常");
	}
}
if (StringUtil.isNotEmpty(map.get("type"))) { 			// 等于空就直接取空值
	try {
		type = (String) map.get("type");
	} catch (Exception e) {
		throw new RuntimeException("type类型转换异常");
	}
}
if (StringUtil.isNotEmpty(map.get("startTime"))) { // 等于空就直接取空值
	try {
		startTime = Long.parseLong((String) map.get("startTime"));
	} catch (Exception e) {
		throw new RuntimeException("startTime类型转换异常");// 数据解析异常或转换类型异常startTime还是0
	}
}
if (StringUtil.isNotEmpty(map.get("endTime"))) { // 等于空就直接取空值
	try {
		endTime = Long.parseLong((String) map.get("endTime"));
	} catch (Exception e) {
		throw new RuntimeException("endTime类型转换异常");// 数据解析异常或转换类型异常endTime还是9223372036854775807L
	}
}*/
}
